local LHR = select(2, ...)

-- Enable drag & drop for all healer frames
function LHR:toggleListSorting(allowSorting)
    for key,healer in pairs(LHR.healerTable) do
        LHR:toggleHealerFrameDragging(healer, allowSorting)
    end
end

-- Enable or disable drag & drop for the healer frame
function LHR:toggleHealerFrameDragging(healer, allowSorting)
    healer.frame:EnableMouse(allowSorting)
    healer.frame:SetMovable(allowSorting)
end

-- configure healer frame drag behavior
function LHR:configureHealerFrameDrag(healer)

    healer.frame:RegisterForDrag("LeftButton")
    healer.frame:SetClampedToScreen(true)

    healer.frame:SetScript(
        "OnDragStart",
        function()
            healer.frame:StartMoving()
            healer.frame:SetFrameStrata("HIGH")
            LHR.mainFrame.rulerFrame:SetPoint('BOTTOMRIGHT', healer.frame, 'TOPLEFT', 0, 0)
            LHR.mainFrame.dropHintFrame:Show()
            LHR.mainFrame.backupFrame:Show()
        end
    )

    healer.frame:SetScript(
        "OnDragStop",
        function()
            healer.frame:StopMovingOrSizing()
            healer.frame:SetFrameStrata(LHR.mainFrame:GetFrameStrata())
            LHR.mainFrame.dropHintFrame:Hide()

            if (#LHR.rotationTables.backup < 1) then
                LHR.mainFrame.backupFrame:Hide()
            end

            local group, position = LHR:getDropPosition(LHR.mainFrame.rulerFrame:GetHeight())
            LHR:handleDrop(healer, group, position)
            LHR:sendSyncOrder(false)
        end
    )
end

-- create and initialize the drop hint frame
function LHR:createDropHintFrame()

    local hintFrame = CreateFrame("Frame", nil, LHR.mainFrame.rotationFrame)

    hintFrame:SetPoint('TOP', LHR.mainFrame.rotationFrame, 'TOP', 0, 0)
    hintFrame:SetHeight(LHR.constants.healerFrameHeight)
    hintFrame:SetWidth(LHR.constants.mainFrameWidth - 10)

    hintFrame.texture = hintFrame:CreateTexture(nil, "BACKGROUND")
    hintFrame.texture:SetColorTexture(LHR.colors.white:GetRGB())
    hintFrame.texture:SetAlpha(0.7)
    hintFrame.texture:SetPoint('LEFT')
    hintFrame.texture:SetPoint('RIGHT')
    hintFrame.texture:SetHeight(2)

    hintFrame:Hide()

    LHR.mainFrame.dropHintFrame = hintFrame
end

-- Create and initialize the 'ruler' frame.
-- It's height will be used as a ruler for position calculation
function LHR:createRulerFrame()

    local rulerFrame = CreateFrame("Frame", nil, LHR.mainFrame.rotationFrame)
    LHR.mainFrame.rulerFrame = rulerFrame

    rulerFrame:SetPoint('TOPLEFT', LHR.mainFrame.rotationFrame, 'TOPLEFT', 0, 0)

    rulerFrame:SetScript(
        "OnSizeChanged",
        function (self, width, height)
            LHR:setDropHintPosition(self, width, height)
        end
    )

end

-- Set the drop hint frame position to match dragged frame position
function LHR:setDropHintPosition(self, width, height)

    local healerFrameHeight = LHR.constants.healerFrameHeight
    local healerFrameSpacing = LHR.constants.healerFrameSpacing
    local hintPosition = 0

    local group, position = LHR:getDropPosition(height)

    if (group == 'ROTATION') then
        if (position == 0) then
            hintPosition = -2
        else
            hintPosition = (position) * (healerFrameHeight + healerFrameSpacing) - healerFrameSpacing / 2;
        end
    else
        hintPosition = LHR.mainFrame.rotationFrame:GetHeight()

        if (position == 0) then
            hintPosition = hintPosition - 2
        else
            hintPosition = hintPosition + (position) * (healerFrameHeight + healerFrameSpacing) - healerFrameSpacing / 2;
        end
    end

    LHR.mainFrame.dropHintFrame:SetPoint('TOP', 0 , -hintPosition)
end

-- Compute drop group and position from ruler height
function LHR:getDropPosition(rulerHeight)

    local group = 'ROTATION'
    local position = 0

    local healerFrameHeight = LHR.constants.healerFrameHeight
    local healerFrameSpacing = LHR.constants.healerFrameSpacing

    -- Dragged frame is above rotation frames
    if (LHR.mainFrame.rulerFrame:GetTop() > LHR.mainFrame.rotationFrame:GetTop()) then
        rulerHeight = 0
    end

    position = floor(rulerHeight / (healerFrameHeight + healerFrameSpacing))

    -- Dragged frame is bellow rotation frame
    if (rulerHeight > LHR.mainFrame.rotationFrame:GetHeight()) then

        group = 'BACKUP'

        -- Removing rotation frame size from calculation, using it's height as base hintPosition offset
        rulerHeight = rulerHeight - LHR.mainFrame.rotationFrame:GetHeight()

        if (rulerHeight > LHR.mainFrame.backupFrame:GetHeight()) then
            -- Dragged frame is bellow backup frame
            position = #LHR.rotationTables.backup
        else
            position = floor(rulerHeight / (healerFrameHeight + healerFrameSpacing))
        end
    end

    return group, position
end

-- Compute the table final position from the drop position
function LHR:handleDrop(healer, group, position)

    local originTable = LHR:getHealerRotationTable(healer)
    local originIndex = LHR:getHealerIndex(healer, originTable)

    local destinationTable = LHR.rotationTables.rotation
    local finalPosition = 1

    if (group == "BACKUP") then
        destinationTable = LHR.rotationTables.backup
    end

    if (destinationTable == originTable) then

        if (position == originIndex or position == originIndex - 1 ) then
            finalPosition = originIndex
        else
            if (position > originIndex) then
                finalPosition = position
            else
                finalPosition = position + 1
            end
        end

    else
        finalPosition = position + 1
    end

    LHR:moveHealer(healer, group, finalPosition)
end

-- Update drag and drop status to match player status
function LHR:updateDragAndDrop()
    LHR:toggleListSorting(LHR:isPlayerAllowedToSortHealerList())
end
