local LHR = select(2, ...)

local debuff = GetSpellInfo(29199)
local testSpell1 = GetSpellInfo(25315)
local testSpell2 = GetSpellInfo(25299)
local testSpell3 = GetSpellInfo(17)
local testSpell4 = GetSpellInfo(1022)

local eventFrame = CreateFrame("Frame")
eventFrame:RegisterEvent("PLAYER_LOGIN")
eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
eventFrame:RegisterEvent("GROUP_ROSTER_UPDATE")
eventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")

eventFrame:SetScript(
    "OnEvent",
    function(self, event, ...)
        if (event == "PLAYER_LOGIN") then
            LHR:init()
            self:UnregisterEvent("PLAYER_LOGIN")

            -- Delayed raid update because raid data is unreliable at PLAYER_LOGIN
            C_Timer.After(5, function()
                LHR:updateRaidStatus()
            end)
        else
            LHR[event](LHR, ...)
        end
    end
)

function LHR:isDebuff(spellName)

	if(spellName ~= nil and spellName == debuff) then
		return true
	end
	return false
end

function LHR:isTestSpell(spellName)

	if(spellName ~= nil and spellName == testSpell1 or spellName == testSpell2 or spellName == testSpell3 or spellName == testSpell4) then
		return true
	end
	return false
end

function LHR:COMBAT_LOG_EVENT_UNFILTERED()

    if not LHR.raidInitialized then return end
    if not LHR.testMode and not IsInInstance() then return end

    local timestamp, event, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags = CombatLogGetCurrentEventInfo()
    local spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing, isOffHand = select(12, CombatLogGetCurrentEventInfo())
	
	local healer = LHR:getHealer(nil, sourceGUID)
	if healer == nil then return end
	
    if (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESHED") then
        if (LHR:isDebuff(spellName) or (LHR.testMode and LHR:isTestSpell(spellName))) then
		
		    LHR:sendSyncHeal(healer, timestamp)
            LHR:rotate(healer, false)
			
			if  (sourceGUID == UnitGUID("player")) then
                LHR:sendAnnounceMessage(LHR.db.profile.announceSuccessMessage, destName)
            elseif (LHR:isPlayerNextHeal()) then
				LHR:throwHealAlert()
			end
		end
    end
end

-- Raid group has changed
function LHR:GROUP_ROSTER_UPDATE()
    LHR:updateRaidStatus()
end

-- Player left combat
function LHR:PLAYER_REGEN_ENABLED()
    LHR:updateRaidStatus()
end

-- Register single unit events for a given healer
function LHR:registerUnitEvents(healer)

    healer.frame:RegisterUnitEvent("PARTY_MEMBER_DISABLE", healer.name)
    healer.frame:RegisterUnitEvent("PARTY_MEMBER_ENABLE", healer.name)
    healer.frame:RegisterUnitEvent("UNIT_HEALTH", healer.name)
    healer.frame:RegisterUnitEvent("UNIT_CONNECTION", healer.name)
    healer.frame:RegisterUnitEvent("UNIT_FLAGS", healer.name)

    healer.frame:SetScript(
        "OnEvent",
        function(self, event, ...)
            LHR:updateHealerStatus(healer)
        end
    )

end

-- Unregister single unit events for a given healer
function LHR:unregisterUnitEvents(healer)
    healer.frame:UnregisterEvent("PARTY_MEMBER_DISABLE")
    healer.frame:UnregisterEvent("PARTY_MEMBER_ENABLE")
    healer.frame:UnregisterEvent("UNIT_HEALTH_FREQUENT")
    healer.frame:UnregisterEvent("UNIT_CONNECTION")
    healer.frame:UnregisterEvent("UNIT_FLAGS")
end

-- Handle timed alert for non healed
function LHR:handleTimedAlert()
    if (LHR.db.profile.enableTimedBackupAlertValue) then
        C_Timer.After(LHR.db.profile.timedBackupAlertValueDelay, function()
            if (LHR:isPlayerNextHeal()) then
                LHR:alertBackup(LHR.db.profile.unableToHealMessage)
            end
        end)
    end
end
